@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2025 Meltytech, LLC
+ * Copyright (c) 2014-2026 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +42,7 @@
 #include <QMediaDevices>
 #include <QMessageBox>
 #include <QProcess>
+#include <QProcessEnvironment>
 #include <QRegularExpression>
 #include <QStandardPaths>
 #include <QStorageInfo>
@@ -59,9 +60,9 @@
 #endif
 
 #ifdef Q_OS_MAC
-static const unsigned int kLowMemoryThresholdPercent = 10U;
+static constexpr unsigned int kLowMemoryThresholdPercent = 10U;
 #else
-static const unsigned int kLowMemoryThresholdKB = 256U * 1024U;
+static constexpr unsigned int kLowMemoryThresholdKB = 256U * 1024U;
 #endif
 static const qint64 kFreeSpaceThesholdGB = 25LL * 1024 * 1024 * 1024;
 
@@ -127,7 +128,7 @@ void Util::showInFolder(const QString &path)
     if (!QProcess::execute("/usr/bin/osascript", args))
         return;
 #endif
-    QDesktopServices::openUrl(QUrl::fromLocalFile(info.isDir() ? path : info.path()));
+    Util::openUrl(QUrl::fromLocalFile(info.isDir() ? path : info.path()));
 }
 
 bool Util::warnIfNotWritable(const QString &filePath, QWidget *parent, const QString &caption)
@@ -466,10 +467,18 @@ bool Util::hasDriveLetter(const QString &path)
     return driveSeparators == ":/" || driveSeparators == ":\\";
 }
 
+QColorDialog::ColorDialogOptions Util::getColorDialogOptions()
+{
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+    return QColorDialog::DontUseNativeDialog;
+#endif
+    return QColorDialog::ColorDialogOptions();
+}
+
 QFileDialog::Options Util::getFileDialogOptions()
 {
 #if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
-    if (qEnvironmentVariableIsSet("SNAP") || qEnvironmentVariableIsSet("GNOME_SHELL_SESSION_MODE")) {
+    if (qEnvironmentVariableIsSet("SNAP")) {
         return QFileDialog::DontUseNativeDialog;
     }
 #endif
@@ -996,6 +1005,11 @@ QPair<bool, bool> Util::dockerStatus(const QString &imageName)
 {
     // Check if docker executable is available by running 'docker --version'.
     QProcess proc;
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+    auto env = QProcessEnvironment::systemEnvironment();
+    env.remove("LD_LIBRARY_PATH");
+    proc.setProcessEnvironment(env);
+#endif
     proc.start(Settings.dockerPath(), {"--version"});
     if (!proc.waitForStarted(1000)) {
         return qMakePair(false, false);
@@ -1016,6 +1030,11 @@ QPair<bool, bool> Util::dockerStatus(const QString &imageName)
     // Query local images for the given name (may include tag). We avoid pulling.
     // Use 'docker image inspect <imageName>' which returns 0 if present.
     QProcess procImage;
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+    env = QProcessEnvironment::systemEnvironment();
+    env.remove("LD_LIBRARY_PATH");
+    procImage.setProcessEnvironment(env);
+#endif
     procImage.start(Settings.dockerPath(), {"image", "inspect", imageName});
     if (!procImage.waitForStarted(1000)) {
         return qMakePair(true, false);
@@ -1056,6 +1075,11 @@ bool Util::isDockerImageCurrent(const QString &imageRef)
 
     // Inspect (local/remote combined behavior) - this is a simplistic approach.
     QProcess localProc;
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+    auto env = QProcessEnvironment::systemEnvironment();
+    env.remove("LD_LIBRARY_PATH");
+    localProc.setProcessEnvironment(env);
+#endif
     localProc.start(Settings.dockerPath(), {"image", "inspect", imageRef});
     if (!localProc.waitForStarted(1000) || !localProc.waitForFinished(4000)) {
         if (localProc.state() != QProcess::NotRunning)
@@ -1068,6 +1092,11 @@ bool Util::isDockerImageCurrent(const QString &imageRef)
 
     // Run again expecting remote freshness (will hit registry) without pulling.
     QProcess remoteProc;
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+    env = QProcessEnvironment::systemEnvironment();
+    env.remove("LD_LIBRARY_PATH");
+    remoteProc.setProcessEnvironment(env);
+#endif
     remoteProc.start(Settings.dockerPath(), {"manifest", "inspect", imageRef});
     if (!remoteProc.waitForStarted(1000) || !remoteProc.waitForFinished(15000)) {
         if (remoteProc.state() != QProcess::NotRunning)
@@ -1110,6 +1139,13 @@ void Util::isDockerImageCurrentAsync(const QString &imageRef,
 
     // Start with local inspect.
     auto *localProc = new QProcess(receiver);
+
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+    auto env = QProcessEnvironment::systemEnvironment();
+    env.remove("LD_LIBRARY_PATH");
+    localProc->setProcessEnvironment(env);
+#endif
+
     QObject::connect(localProc,
                      &QProcess::errorOccurred,
                      localProc,
@@ -1129,6 +1165,13 @@ void Util::isDockerImageCurrentAsync(const QString &imageRef,
             localProc->deleteLater();
             // Remote manifest inspect regardless; if docker missing it will fail.
             auto *remoteProc = new QProcess(receiver);
+
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+            auto env = QProcessEnvironment::systemEnvironment();
+            env.remove("LD_LIBRARY_PATH");
+            remoteProc->setProcessEnvironment(env);
+#endif
+
             QObject::connect(remoteProc,
                              &QProcess::errorOccurred,
                              remoteProc,
@@ -1164,3 +1207,56 @@ bool Util::isChromiumAvailable()
     QFileInfo fileInfo(Settings.chromiumPath());
     return fileInfo.exists() && fileInfo.isExecutable();
 }
+
+bool Util::startDetached(const QString &program, const QStringList &arguments)
+{
+    QProcess process;
+
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+    // Remove parts of environment variables that our launch script has set
+    auto env = QProcessEnvironment::systemEnvironment();
+    // Get the parent of bin/shotcut
+    QString appDir = QFileInfo(QCoreApplication::applicationDirPath()).dir().absolutePath();
+
+    auto filterEnvVar = [&env, &appDir](const QString &varName) {
+        QString value = env.value(varName);
+        if (!value.isEmpty()) {
+            QStringList paths = value.split(':');
+            QStringList filtered;
+            for (QString &path : paths) {
+                if (!path.contains(appDir)) {
+                    filtered << path;
+                } else {
+                    LOG_DEBUG() << "removing" << path << "from env var" << varName;
+                }
+            }
+            env.insert(varName, filtered.join(':'));
+        }
+    };
+
+    filterEnvVar("FREI0R_PATH");
+    filterEnvVar("LADSPA_PATH");
+    filterEnvVar("LD_LIBRARY_PATH");
+    filterEnvVar("MANPATH");
+    filterEnvVar("PKG_CONFIG_PATH");
+    filterEnvVar("PYTHONHOME");
+
+    // These are relative paths and not cleanable with filterEnvVar()
+    env.remove("QML2_IMPORT_PATH");
+    env.remove("QT_PLUGIN_PATH");
+
+    process.setProcessEnvironment(env);
+#endif
+
+    return process.startDetached(program, arguments);
+}
+
+bool Util::openUrl(const QUrl &url)
+{
+    auto success = QDesktopServices::openUrl(url);
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+    if (!success)
+        success = startDetached("xdg-open", {url.toString()});
+#endif
+    return success;
+}
